import math
from readability.text.analyzer import Analyzer
from readability.exceptions import ReadabilityException
import warnings

class Result:
    def __init__(self, score, grade_level):
        self.score = score
        self.grade_level = grade_level

    def __str__(self):
        return "score: {}, grade_level: {}". \
            format(self.score, self.grade_level)


class Smog:
    def __init__(self, stats, sentences, all_sentences=False, ignore_length=False):
        """
        Computes the SMOG readability score (Harry McLaughlin, 1969 https://ogg.osu.edu/media/documents/health_lit/WRRSMOG_Readability_Formula_G._Harry_McLaughlin__1969_.pdf)
        If all_sentences is false, computes the score as described in McLaughlin, 1969, using exactly 30 sentences
        If all_sentences is true, adjusts the score to use all sentences in the text
        """
        if stats.num_sentences < 30:
            if not ignore_length:
                raise ReadabilityException(
                    'SMOG requires 30 sentences. {} found'
                    .format(stats.num_sentences))
            else:
                warnings.warn(
                    'SMOG requires 30 sentences. {} found'
                    .format(stats.num_sentences))


        self._stats = stats
        self.all_sentences = all_sentences
        if not self.all_sentences:
            self._smog_stats = self._smog_text_stats(sentences)
            

    def score(self):
        score = self._score()
        grade_level = self._grade_level(score)
        return Result(
            score=score,
            grade_level=grade_level
        )

    def _score(self):
        if self.all_sentences:
            smog_stats = self._stats
            num_sentences = smog_stats.num_sentences
        else:
            smog_stats = self._smog_stats
            num_sentences = 30
        
        num_complex_words = smog_stats.num_poly_syllable_words
        return 1.0430 * math.sqrt(30 * num_complex_words / num_sentences) + 3.1291

    def _grade_level(self, score):
        return str(round(score))

    def _smog_text_stats(self, sentences):
        mid = int(math.floor(len(sentences) / 2))
        first_10_sents = sentences[:10]
        mid_10_sents = sentences[mid-5: mid+5]
        last_10_sents = sentences[-10:]

        smog_text = ' '.join(first_10_sents + mid_10_sents + last_10_sents)
        return Analyzer().analyze(smog_text)



from .text import Analyzer
from .scorers import ARI, ColemanLiau, DaleChall, Flesch, \
    FleschKincaid, GunningFog, LinsearWrite, Smog, Spache
import warnings

class Readability:
    def __init__(self, text, min_words=0):
        self._analyzer = Analyzer()
        self._statistics = self._analyzer.analyze(text)
        self._min_words = min_words
        # if self._min_words < 100:
        #     warnings.warn(
        #         "Documents with fewer than 100 words may affect the accuracy of readability tests"
        #     )

    def ari(self):
        """Calculate Automated Readability Index (ARI)."""
        return ARI(self._statistics, self._min_words).score()

    def coleman_liau(self):
        """Calculate Coleman Liau Index."""
        return ColemanLiau(self._statistics, self._min_words).score()

    def dale_chall(self):
        """Calculate Dale Chall."""
        return DaleChall(self._statistics, self._min_words).score()

    def flesch(self):
        """Calculate Flesch Reading Ease score."""
        return Flesch(self._statistics, self._min_words).score()

    def flesch_kincaid(self):
        """Calculate Flesch-Kincaid Grade Level."""
        return FleschKincaid(self._statistics, self._min_words).score()

    def gunning_fog(self):
        """Calculate Gunning Fog score."""
        return GunningFog(self._statistics, self._min_words).score()

    def linsear_write(self):
        """Calculate Linsear Write."""
        return LinsearWrite(self._statistics, self._min_words).score()

    def smog(self,all_sentences=False, ignore_length=False):
        """SMOG Index.
        `all_sentences` indicates whether SMOG should use a sample of 30 sentences, as described in the original paper, or if it should use all sentences in the text"""
        return Smog(self._statistics, self._analyzer.sentences,
                    all_sentences=all_sentences, ignore_length=ignore_length).score()

    def spache(self):
        """Spache Index."""
        return Spache(self._statistics, self._min_words).score()

    def statistics(self):
        return {
            'num_letters': self._statistics.num_letters,
            'num_words': self._statistics.num_words,
            'num_sentences': self._statistics.num_sentences,
            'num_polysyllabic_words': self._statistics.num_poly_syllable_words,
            'avg_words_per_sentence': self._statistics.avg_words_per_sentence,
            'avg_syllables_per_word': self._statistics.avg_syllables_per_word,
        }
